#############################################################################################################################
# Filename: constant.R
# Author:	Simon Barry & Peter Caley (CSIRO computational informatics)
#############################################################################################################################
library(MASS)

#############################################################################################################################
# Tasmanian fox carcass discovery data as of end 2012
y.2012 <- c(1,0,1,0,1,1,0,0,0,0,0,0)

# Hyper priors for lambda and phi
lam.wp <- c(1.1,1.1)	# wp for "weak prior"
phi.wp <- c(1.1,1.1)

# number of iterations
n.iter <- 1E5
#n.iter <- 1E6

#############################################################################################################################
fit.func <- function(y, phi1, phi2, lambda1, lambda2, iter=1E5, initphi=.5, initlambda=.5)
# Args:
# 	y -- observation data vector 1=observed that time period, 0 not observed
# 	phi1, phi2 -- beta parameters for extinction probablity prior
# 	lambda1, lambda2 -- beta parameters for detection probability prior
# 	initphi -- initial value for yearly survival probability
# 	initlambda -- initial value for yearly detection probability (given extant)
# Returns:
#	matrix containing Gibbs samples for phi, lambda and z (extinction year) 
{
    # determine last recorded detection
    pos <- (1:length(y))[y==1]

    # year of most recent detection
    final <- pos[length(pos)]				
    
    # how many observation years
    obs <- length(y)					
    
    # setup
    curr.phi <- initphi
    curr.lambda <- initlambda

    # number of observation periods since last detection
    periods <- obs-final				
    result <- matrix(0, nrow=iter, ncol=3)
   
    #sampler
    for(i in 1:iter) {
        newz <- impute(curr.phi, curr.lambda, periods)
        index <- (1:length(newz))[newz==1]
        extra.y <- NA


	  # If extinction is beyond observation period (right-censored)
        if (index > periods)  {   			

		   # Estimate extra time before extinction assuming Geometric (using memoryless property of Geometric distn)
		   extra.z <- rgeom(1,curr.phi) + 1 

		   # Impute time of extinction	
               curr.z <- obs + extra.z

		   # Generate future detections
               extra.y <- rbinom(extra.z-1,1,curr.lambda)	

	  } else {
                curr.z <- final + index
        }

	  # Draw samples for phi and lambda	
        curr.phi <- rbeta(1, phi1 + 1, phi2 + curr.z - 1)
        curr.lambda <- rbeta(1, lambda1 + sum(c(y,extra.y),na.rm=T), lambda2 + curr.z - sum(c(y,extra.y),na.rm=T))

        result[i,] <- c(curr.phi, curr.lambda, curr.z)
    }
    result
}

impute <- function(phi, lambda, periods) 
# Args:
# 	phi -- extinction probability
# 	lambda -- detection probability (given extant)
# 	periods -- observations since last detection
# Returns:
# 	vector giving "position" of extinction since last sighting. Last position is right-censored
{
    result <- 1:(periods+1)
    for(i in 1:periods) {
     result[i] <- phi*((1-phi)*(1-lambda))^(i-1)
    }
    # Extinction beyond observation period
    result[periods+1] <- exp(periods*log((1-phi)*(1-lambda)))	

    # Normalize 
    result <- result/sum(result)

    # Draw extinction 'bin'
    answer <- rmultinom(1,1,result)
    answer
}


##################################################################################
# Run sampler

# 2001 incursion start
res.wp <- fit.func(y=y.2012,phi1=phi.wp[1],phi2=phi.wp[2],lambda1=1.1,lambda2=lam.wp[2],iter=n.iter)		

# 1999 incursion start
res.pp.wp <- fit.func(y=c(c(0,0),y.2012),phi1=phi.wp[1],phi2=phi.wp[2],lambda1=lam.wp[1],lambda2=lam.wp[2],iter=n.iter)	

##################################################################################
# Quick look at posteriors & priors
par(mfrow=c(2,1),cex.lab=2)
truehist(res.wp[,1],xlab=expression(phi),xlim=c(0,1))
curve(dbeta(x,phi.wp[1],phi.wp[2]),add=T,lwd=2,lty=2)
truehist(res.wp[,2],xlab=expression(lambda),xlim=c(0,1))
curve(dbeta(x,lam.wp[1],lam.wp[2]),add=T,lwd=2,lty=2)

##################################################################################
