################################################################################
# Filename: non_constant.R
# Author:	Peter Caley (CSIRO Computational Informatics, E:peter.caley@csiro.au)
# Changes:	Initial N fixed at 1 as N and delta not uniquely estimable
################################################################################
library(MASS)

# Set burn in and length of chain
burn.in <- 1E3
n.iter <- 1E4 + burn.in
 
# burn.in <- 1E5
# n.iter <- 1E6 + burn.in

# Specify year to end observation process
year.end <- 2012

################################################################################
# Sighting data
dat.end.2012 <- c(1,0,1,0,1,1,0,0,0,0,0,0)
dat.end.2013 <- c(1,0,1,0,1,1,0,0,0,0,0,0,0)	# Not run yet

################################################################################
sim.N <- function(pgr,time,N0) {
# Returns:
#	population trajectory from start of observation period
# Args:
#	N0 - initial population size
#	pgr - exponential population growth rate per year
#	time - time to end of observation period
 ans <- numeric(time)
 ans[1] <- N0	
 if(time>1) {
  for (t in 2:time){
   ans[t] <- ans[t-1]*exp(pgr)
  }
 } 
 ans
}

logit <- function(x) {
# Returns:
#	logit transformation
 log(x/(1-x))
}

ilogit <- function(x)  {
# Returns:
#	inverts logit transformation	
 1/(1+exp(-x))
}

lam.gen <- function(delta,N) {
# Returns:
# 	vector of yearly detection probability based on population size
# Args: 
# 	delta -- per capita detection rate 
# 	N -- population size
  1-exp(-delta*N)	
}

phi.gen <- function(eps0,eps1,N) {
# Returns:
# 	vector of extinction probability based on population size
# Args: 
#	eps0 -- intercept for logit of extinction probability
#	eps1 -- population coefficient for logit of extinction probability

  lp <- eps0 - eps1*log(N)
  # Avoiding numerical overflow
  if (lp>50) {
      ans <- 1
     } else {
   	ans <- ilogit(lp)
  }
  ans
}


correct.negative <- function(x) {
# For correcting numerical errors for non-negative vectors
 x[sign(x)<0] <- 0
 x
}

impute.TE <- function(p.cease) {
# Returns:
#	 Estimated time to extinction (TE) within observation window, o.w. returns NA if right-censored
# Args:
# 	p.cease -- a vector of yearly estimates of extinction during the observation window (p.cease) 

	# get rid of any floating point errors
	p.cease <- correct.negative(p.cease)	

	# Probability of being right-censored
	p.cens <- tail(p.cease,1)
	if(sign(p.cens<0)) p.cens <- 0

	# Random draw for time to extinction conditional on p.cease
	TE.draw <- rmultinom(1,1,p.cease)
	
	# Where does extinction occur
	result <- (1:length(p.cease))[TE.draw==1]	
	if (result <= (length(p.cease)-1)) {
		return(result)
	} else {
		return(NA)
	}
}

calc.pars <- function(p, y=dat.end.2012) {		 
# Args: 
# 	 p - c(pgr (pop. growth rate), delta, eps0, eps1, N0)
# Function calls: 
#	sim.N(), lam.gen(), phi.gen(), impute.TE() 
# Returns:
#	y.obs -- observations up until imputed time of extinction
#	N -- vector containing population trajctory
#	phis -- vector of extinction probabilities
#	lambdas -- vector of detection probabilities	
#	p.cease -- vector of probability of extinction over the zero.count period and beyond
#	p.cens -- probability of right-censoring in extinction year [also present as last entry in p.cease]
# 	TE -- imputed time of extinction from impute.TE()

 pgr <-  p[1]
 delta <- p[2]
 eps0 <- p[3] 
 eps1 <- p[4]
 N0 <- p[5]
 obs <- length(y)

 # positions of sightings
 pos <- (1:length(y))[y==1]		

 # time of last sighting [from beginning of observation period]
 final <- tail(pos,1)			

 # observations since last sighting
 zero.count <- obs - final		
 
 # projected population trajectory up until end of current observation period
 N.traj <- sim.N(N0=N0, pgr=pgr, time=obs)			

 # vector of detection probs
 lambdas <- sapply(N.traj, function(x) lam.gen(delta=delta,N=x))	

 # vector of extinction probs
 phis <- sapply(N.traj, function(x) phi.gen(eps0=eps0,eps1=eps1,N=x))	

 # P(extinction) by year following last sighting "+1" for right censoring
 p.cease <- numeric(obs+1)		

 # Can't have ceased prior to last sighting
 p.cease[1:final] <- 0			

 # First extinction probability following last sighting is simply phi for that time
 p.cease[final+1] <- phis[final+1]		 
 if(zero.count>1) {
	 # for the remaining period of no sightings	 
	 for (d in (final+2):obs) {							
	   log.ans <- sum(log(1-phis[(final+1):(d-1)])) + sum(log(1-lambdas[(final+1):(d-1)])) + log(phis[d])
	   p.cease[d] <- exp(log.ans)
	  #p.cease[d] <- prod(1-phis[(final+1):(d-1)])*prod(1-lambdas[(final+1):(d-1)])*phis[d] 	# potentially numerically challenging
	 }
 }
 # get rid of any floating point errors
 p.cease <- correct.negative(p.cease)	

 # calculating probability of right-censoring for TE - Not extinct & not seen for zero.count

 if(sum(p.cease)>0) {
   log.ans <- sum(log(1-phis[(final+1):obs])) + sum(log(1-lambdas[(final+1):obs]))
   p.cease[obs+1] <- exp(log.ans)
  #p.cease[obs+1] <- prod(1-phis[(final+1):obs])*prod(1-lambdas[(final+1):obs])		# potentially numerically challenging
  } else {
   p.cease[obs+1] <- 1				# right-censored for sure as zero prob. mass for TE<T
 } 
 # select only observation data for period following last sighting
 p.cease <- p.cease[(final+1):(final+zero.count+1)]	
 # Normalizee
 p.cease <- p.cease/sum(p.cease)		
 p.cens <- tail(p.cease,1)

 # Calculate imputed extinction time [from beginning of observation period]
 TE <- final + impute.TE(p.cease)   	

 if(!is.na(TE)) {					
 # TE occurs during observation period
	phis <- head(phis,TE)		# Trim to length of TE
	lambdas <- head(lambdas,TE)	# Trim to length of TE
	y.obs <- head(y,TE)		# Trim to length of TE
	N.traj <- head(N.traj,TE) 	# Trim to length of TE
 } else {
 # If right-censored then all observations up until end of observation period are included
	y.obs <- y				
 }
 list(N=N.traj, p.cease=p.cease, p.cens=p.cens, TE=TE, y.obs=y.obs, phis=phis, lambdas=lambdas )
}

calc.pars.given.TE <- function(p,TE,y=y) {	# p=prop.p TE=TE.imp
# Function to calculate the phis and the lambdas conditional on TE
# Args: 
#  N0 - intial population size
#  p - proposed pars (pgr,delta,eps0)
#  y - observed data
# Function calls: sim.N(), phi.gen(), lam.gen()
 pgr <-  p[1]
 delta <- p[2]
 eps0 <- p[3] 
 eps1 <- p[4]
 N0 <- p[5]
 
 if(!is.na(TE)) {
   # population trajectory until extinction prior to end of observation period
   N.traj <- sim.N(N0=N0, pgr=pgr, time=TE)							
  } else {
   # population trajectory until right censoring
   N.traj <- sim.N(N0=N0, pgr=pgr, time=length(y))						
 }

 # detection probs until right censoring
 lambdas <- sapply(N.traj, function(x) lam.gen(delta=delta,N=x))				

 # extinction probs until right censoring 
 phis <- sapply(N.traj, function(x) phi.gen(eps0=eps0,eps1=eps1,N=x))			

 list(N=N.traj, TE=TE, phis=phis, lambdas=lambdas)
}

# Log likelihood for lamba
lnL.lam <- function(lams,ys){
	sum(log(lams[ys!=0])) + sum(log(1-lams[ys!=1]))
}

# Log likelihood for phi
lnL.phi <- function(phis,TE) {
 if(!is.na(TE)) {			
	# Extinct before end of observation period
 	ans <- sum(log(1-phis[-length(phis)])) + log(tail(phis,1))
 } else {				
 	# Right-censored (extinction beyond observation period)
	ans <- sum(log(1-phis))
 }
 ans
}

###############################################################################################################
# Specify prior functions	
N0.prior <- function(x) dunif(x,5,50,log=F)				
lnL.N0.prior <- function(x) dunif(x,5,50,log=T)		

pgr.prior <- function(x) dunif(x,min=-2.3, max=0.69,log=F)		 
lnL.pgr.prior <- function(x) dunif(x,min=-2.3, max=0.69,log=T)	 
		
delta.prior <- function(x) dunif(x,0.01,4.6,log=F)
lnL.delta.prior <- function(x) dunif(x,0.01,4.6,log=T)	 

eps0.bounds <- c(-20,20)
eps0.prior <- function(x) dunif(x,min(eps0.bounds),max(eps0.bounds),log=F)			
lnL.eps0.prior <- function(x) dunif(x,min(eps0.bounds),max(eps0.bounds),log=T)			

eps1.bounds <- c(0,20) 
eps1.prior <- function(x) dunif(x,min(eps1.bounds),max(eps1.bounds),log=F)
lnL.eps1.prior <- function(x) dunif(x,min(eps1.bounds),max(eps1.bounds),log=T)

################################################################################################################
# Proposal functions

# Proposals shared amongst models
propose.N0 <- function(x){
	return(x)}
propose.pgr <- function(x, sd=0.05) {		
	rnorm(1,x,sd=sd)}
propose.delta <- function(x,sd=0.5) {
	rlnorm(1,log(x),sd)}
q.delta <- function(x1,x2) {dlnorm(x1,log(x2),sd=0.5,log=T)} 

propose.eps0 <- function(x, sd=1.5) {
	rnorm(1,x,sd=sd)}
propose.eps1 <- function(x, sd=1.5) {
	rnorm(1,x,sd=sd)} 

propose.eps1 <- function(x,sd=0.15) {
 	rlnorm(1,log(x),sd)}
q.eps1 <- function(x1,x2) {dlnorm(x1,log(x2),sd=0.15,log=T)} 

################################################################################################################
# Function to run MCMC sampler with MH 
fit.func <- function(N0.init=1,y=dat.end.2012,iter=100, pgr.init=0.0, delta.init=NA, eps0.init=0.0, eps1.init=-0.1) {	
  
  # NB "prop" is shorthand for proposed
  # NB "curr" is shorthand for current
 
  # Set delta.init such that initial lambda is 0.5
  delta.init <- -log(1-0.5)/N0.init

  # Initialize current parameters
  curr.p <- c(pgr.init,delta.init,eps0.init,eps1.init,N0.init)  
 
  # Setup
  result <- matrix(0,nrow=iter,ncol=10)	# pgr, delta, eps0, eps1, N0, TE, acceptance indicators (x4)
  for (i in 1:iter) {	#i=1	i=i+1
   cat("Doing interation", i, "of", iter, "\n") 

   # Generate values of y's & T_E by imputation conditional on current N0, delta, eps0 and eps1 
   curr.vals <- calc.pars(p=curr.p,y=y)

   # imputed value for TE -- stays fixed for remainder of conditional sampling
   TE.imp <- curr.vals$TE			

   # observed observations corresponding to TE.imp
   y.obs <- curr.vals$y.obs			
   
   # Update delta

   curr.lnLik.delta <- lnL.lam(lams=curr.vals$lambdas,ys=y.obs) + lnL.delta.prior(curr.p[2])

   prop.p <- curr.p
   prop.p[2] <- propose.delta(curr.p[2])
   prop.vals <- calc.pars.given.TE(p=prop.p,TE=TE.imp,y=y)	

   prop.lnLik.delta <- lnL.lam(lams=prop.vals$lambdas,ys=y.obs) + lnL.delta.prior(prop.p[2])
  
   LR <- exp(prop.lnLik.delta - curr.lnLik.delta + q.delta(curr.p[2],prop.p[2]) - q.delta(prop.p[2],curr.p[2]))

   if(runif(1) < LR & !is.na(LR)) {
	curr.p <- prop.p
	curr.vals <- prop.vals
	result[i,8] <- 1 	# indicate acceptance
    } else {
	result[i,8] <- 0 	# indicate rejection
   }

   # Update epsilons

   curr.lnLik.eps <-  lnL.phi(phis=curr.vals$phis,TE=TE.imp) + lnL.eps0.prior(curr.p[3]) + lnL.eps1.prior(curr.p[4]) 

   # Update eps0 annd eps1 jointly
   prop.p <- curr.p
   prop.p[3:4] <- c(propose.eps0(curr.p[3]),propose.eps1(curr.p[4]))
   prop.vals <- calc.pars.given.TE(p=prop.p,TE=TE.imp,y=y)	

   prop.lnLik.eps <- lnL.phi(phis=prop.vals$phis,TE=TE.imp) + lnL.eps0.prior(prop.p[3]) + lnL.eps1.prior(prop.p[4]) 
   LR <- exp(prop.lnLik.eps - curr.lnLik.eps + q.eps1(curr.p[4],prop.p[4]) - q.eps1(prop.p[4],curr.p[4]))

   if(runif(1) < LR & !is.na(LR)) {
	curr.p <- prop.p
	curr.vals <- prop.vals
	result[i,9] <- 1 	# indicate acceptance
   } else {
	result[i,9] <- 0 	# indicate rejection
   }

   # Update pgr
   curr.lnLik.pgr <- lnL.lam(lams=curr.vals$lambdas,ys=y.obs) + lnL.phi(phis=curr.vals$phis,TE=TE.imp) + lnL.pgr.prior(curr.p[1])
   prop.p <- curr.p
   prop.p[1] <- propose.pgr(curr.p[1])
   prop.vals <- calc.pars.given.TE(p=prop.p,TE=TE.imp,y=y)	

   prop.lnLik.pgr <- lnL.lam(lams=prop.vals$lambdas,ys=y.obs) + lnL.phi(phis=prop.vals$phis,TE=TE.imp) + lnL.pgr.prior(prop.p[1])
   LR <- exp(prop.lnLik.pgr - curr.lnLik.pgr)

   if(runif(1) < LR & !is.na(LR)) {
	curr.p <- prop.p
	curr.vals <- prop.vals
	result[i,7] <- 1 	# indicate acceptance
   } else {
	result[i,7] <- 0 	# indicate rejection
   }


   # Update N0
   curr.lnLik.N0 <- lnL.lam(lams=curr.vals$lambdas,ys=y.obs) + lnL.phi(phis=curr.vals$phis,TE=TE.imp) + lnL.N0.prior(curr.p[5])
   
   prop.p <- curr.p
   prop.p[5] <- propose.N0(curr.p[5])
   prop.vals <- calc.pars.given.TE(p=prop.p,TE=TE.imp,y=y)	

   prop.lnLik.N0 <- lnL.lam(lams=prop.vals$lambdas,ys=y.obs) + lnL.phi(phis=prop.vals$phis,TE=TE.imp) + lnL.N0.prior(prop.p[5])

   LR <- exp(prop.lnLik.N0 - curr.lnLik.N0) 

   if(runif(1) < LR & !is.na(LR)) {
	curr.p <- prop.p
	curr.vals <- prop.vals
	result[i,10] <- 1 	# indicate acceptance
   } else {
	result[i,10] <- 0 	# indicate rejection
   }

   result[i,1:6] <- c(curr.p,TE.imp)
  } # end of iter loop
 result
}


###############################################################################################################################
# Run sampler
 # With a 2001 incursion start
   res1 <- fit.func(iter=n.iter,y=dat.end.2012, pgr.init=0.0, delta.init=0.69, eps0.init=0, eps1.init=1, N0.init=1)
   res1 <- res1[-(1:burn.in),]	 

 # With a 1999 incursion start
   res2 <- fit.func(iter=n.iter, y=c(rep(0,2),dat.end.2012), pgr.init=0.0, delta.init=069, eps0.init=0, eps1.init=1, N0.init=1)
   res2 <- res2[-(1:burn.in),]	

 # Acceptance rate of unthinned chain
 cat("Acceptance rate for pgr with 2001 start =", mean(res1[,7]),"\n")
 cat("Acceptance rate for delta with 2001 start =", mean(res1[,8]),"\n")
 cat("Acceptance rate for epsilons with 2001 start =", mean(res1[,9]),"\n")
 cat("Acceptance rate for N0 with 2001 start =", mean(res1[,10]),"\n")

 cat("Acceptance rate for pgr with 1999 start =", mean(res2[,7]),"\n")
 cat("Acceptance rate for delta with 1999 start =", mean(res2[,8]),"\n")
 cat("Acceptance rate for epsilons with 1999 start =", mean(res2[,9]),"\n")
 cat("Acceptance rate for N0 with 1999 start =", mean(res2[,10]),"\n")


#####################################################################################################################################################
 # Look at chains (exlcuding N0 which is set to one)
win.graph()
par(mfrow=c(5,1),mar=c(2,2,1,1))
apply(res1[,c(1,2,3,4,6)],2,function(x) plot(ts(x)))

win.graph()
par(mfrow=c(5,1),mar=c(2,2,1,1))
apply(res2[,c(1,2,3,4,6)],2,function(x) plot(ts(x)))

# Have a quick look at posteriors (exlcuding N0 which is set to one)
win.graph()
par(mfrow=c(6,1),mar=c(2,2,1,1))
sapply(c(1,2,3,4,6),function(x) truehist(res1[,x]))

win.graph()
par(mfrow=c(6,1),mar=c(2,2,1,1))
sapply(c(1,2,3,4,6),function(x) truehist(res2[,x]))

# P(Extant)
mean(is.na(res1[,6]))
mean(is.na(res2[,6]))

#####################################################################################################################################################
# Parameter estimates
# pgr
quantile(res1[,1],c(0.025,0.5,0.975))
quantile(res2[,1],c(0.025,0.5,0.975))

# delta
quantile(res1[,2],c(0.025,0.5,0.975))
quantile(res2[,2],c(0.025,0.5,0.975))

#eps0
quantile(res1[,3],c(0.025,0.5,0.975))
quantile(res2[,3],c(0.025,0.5,0.975))

#eps1
quantile(res1[,4],c(0.025,0.5,0.975))
quantile(res2[,4],c(0.025,0.5,0.975))


#####################################################################################################################################################
